/*
 * sxgconf.c: S-YXG50 Configurator
 * written by gocha in the public domain, feel free to use :)
 */

#include <tchar.h>
#include <windows.h>
#include <shlwapi.h>
#include "resource.h"
#include "sxgconf.h"

/* macros */
#define length(a)   (sizeof(a) / sizeof(a[0]))

/* constant definitions */
#if 0
#define SXGXGWDM_DEVICEID   _T("sw\\{f25737c5-f523-44b6-9b0a-c3c400a1e31a}") /* syxg50 download version */
#endif
#define SXGXGWDM_DEVICEID   _T("sw\\{cea6a804-0f0c-4fcf-b30f-cc12d19d3e88}")
#define SXGSETUP_ERRORVAL   0x7fffffff

/* sxgconf main */
int WINAPI _tWinMain(HINSTANCE instance, HINSTANCE preInstance, LPTSTR cmdLine, int showCmd)
{
	TCHAR moduleDir[MAX_PATH];
	TCHAR iniPath[MAX_PATH];
	DWORD sxgsetup_poly;
	DWORD sxgsetup_cpuload;
	DWORD sxgsetup_reverb;
	DWORD sxgsetup_chorus;
	DWORD sxgsetup_variation;
	DWORD sxgsetup_fs;
	HKEY keyHandle;
	TCHAR finishMsg[128];

	/* get module directory */
	GetModuleFileName(GetModuleHandle(NULL), moduleDir, MAX_PATH);
	PathRemoveFileSpec(moduleDir);

	/* get sxgsetup.ini path */
	wsprintf(iniPath, _T("%s\\sxgconf.ini"), moduleDir);

	/* get settings from ini */
	sxgsetup_poly = GetPrivateProfileInt(_T("sxgsetup"), _T("poly"), SXGSETUP_ERRORVAL, iniPath);
	sxgsetup_cpuload = GetPrivateProfileInt(_T("sxgsetup"), _T("cpuload"), SXGSETUP_ERRORVAL, iniPath);
	sxgsetup_reverb = GetPrivateProfileInt(_T("sxgsetup"), _T("reverb"), SXGSETUP_ERRORVAL, iniPath);
	sxgsetup_chorus = GetPrivateProfileInt(_T("sxgsetup"), _T("chorus"), SXGSETUP_ERRORVAL, iniPath);
	sxgsetup_variation = GetPrivateProfileInt(_T("sxgsetup"), _T("variation"), SXGSETUP_ERRORVAL, iniPath);
	sxgsetup_fs = GetPrivateProfileInt(_T("sxgsetup"), _T("fs"), SXGSETUP_ERRORVAL, iniPath);

	/* open sound device key */
	if(RegOpenKeyEx(HKEY_LOCAL_MACHINE, _T("SYSTEM\\CurrentControlSet\\Control\\Class\\{4D36E96C-E325-11CE-BFC1-08002BE10318}"), 0, KEY_ALL_ACCESS, &keyHandle) == ERROR_SUCCESS)
	{
		HKEY deviceKeyHandle;
		TCHAR subKeyName[8];
		DWORD subKeyNameLen;
		DWORD subKeyIndex;
		FILETIME lastWriteTime;

		/* look for syxg50 key */
		subKeyIndex = 0;
		subKeyNameLen = length(subKeyName);
		while(RegEnumKeyEx(keyHandle, subKeyIndex, subKeyName, &subKeyNameLen, NULL, NULL, NULL, &lastWriteTime) == ERROR_SUCCESS)
		{
			if(subKeyNameLen == 4)
			{
				TCHAR deviceId[128];
				DWORD deviceIdLen;

				if(RegOpenKeyEx(keyHandle, subKeyName, 0, KEY_READ, &deviceKeyHandle) == ERROR_SUCCESS)
				{
					LONG queryResult;

					deviceIdLen = length(deviceId);
					queryResult = RegQueryValueEx(deviceKeyHandle, _T("MatchingDeviceId"), NULL, NULL, (LPBYTE) deviceId, &deviceIdLen);
					/* is this syxg50 key? */
					if((queryResult == ERROR_SUCCESS) && (lstrcmpi(deviceId, SXGXGWDM_DEVICEID) == 0))
					{
						HKEY sxgsetupKeyHandle;

						/* open syxg50 setup key */
						if(RegOpenKeyEx(deviceKeyHandle, _T("sxgsetup"), 0, KEY_WRITE, &sxgsetupKeyHandle) == ERROR_SUCCESS)
						{
							if(sxgsetup_poly != SXGSETUP_ERRORVAL)
								RegSetValueEx(sxgsetupKeyHandle, _T("poly"), 0, REG_DWORD, (CONST BYTE*) &sxgsetup_poly, sizeof(DWORD));
							if(sxgsetup_cpuload != SXGSETUP_ERRORVAL)
								RegSetValueEx(sxgsetupKeyHandle, _T("cpuload"), 0, REG_DWORD, (CONST BYTE*) &sxgsetup_cpuload, sizeof(DWORD));
							if(sxgsetup_reverb != SXGSETUP_ERRORVAL)
								RegSetValueEx(sxgsetupKeyHandle, _T("reverb"), 0, REG_DWORD, (CONST BYTE*) &sxgsetup_reverb, sizeof(DWORD));
							if(sxgsetup_chorus != SXGSETUP_ERRORVAL)
								RegSetValueEx(sxgsetupKeyHandle, _T("chorus"), 0, REG_DWORD, (CONST BYTE*) &sxgsetup_chorus, sizeof(DWORD));
							if(sxgsetup_variation != SXGSETUP_ERRORVAL)
								RegSetValueEx(sxgsetupKeyHandle, _T("variation"), 0, REG_DWORD, (CONST BYTE*) &sxgsetup_variation, sizeof(DWORD));
							if(sxgsetup_fs != SXGSETUP_ERRORVAL)
								RegSetValueEx(sxgsetupKeyHandle, _T("fs"), 0, REG_DWORD, (CONST BYTE*) &sxgsetup_fs, sizeof(DWORD));

							RegCloseKey(sxgsetupKeyHandle);
						}

						/* close syxg50 key */
						RegCloseKey(deviceKeyHandle);
						break;
					}
					RegCloseKey(deviceKeyHandle);
				}
			}

			subKeyIndex++;
			subKeyNameLen = length(subKeyName);
		}

		RegCloseKey(keyHandle);
	}

	LoadString(instance, IDS_FINISHED, finishMsg, length(finishMsg));
	MessageBox(NULL, finishMsg, _T("S-YXG50 Configurator"), MB_OK | MB_ICONINFORMATION);
	return 0;
}
